from mayan.apps.appearance.classes import Icon

icon_document_create_multiple = Icon(
    driver_name='fontawesome', symbol='upload'
)
icon_document_file_upload = Icon(
    driver_name='fontawesome', symbol='upload'
)

icon_source_backend_selection = Icon(
    driver_name='fontawesome-dual', primary_symbol='upload',
    secondary_symbol='plus'
)
icon_source_delete = Icon(driver_name='fontawesome', symbol='times')
icon_source_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_source_list = Icon(driver_name='fontawesome', symbol='upload')
icon_source_test = Icon(driver_name='fontawesome', symbol='check')

icon_upload_view_link = Icon(driver_name='fontawesome', symbol='upload')

icon_wizard_step_previous = Icon(driver_name='fontawesome', symbol='caret-left')
icon_wizard_step_first = Icon(driver_name='fontawesome', symbol='step-backward')
icon_wizard_step_next = Icon(driver_name='fontawesome', symbol='caret-right')
