API_VERSION = '4'
DEFAULT_PAGE_SIZE_QUERY_PARAMETER = 'page_size'
DEFAULT_REST_API_DISABLE_LINKS = False
DEFAULT_REST_API_MAXIMUM_PAGE_SIZE = 100
DEFAULT_REST_API_PAGE_SIZE = 10

QUERY_FIELD_EXCLUDE_PARAMETER = '_fields_exclude'
QUERY_FIELD_ONLY_PARAMETER = '_fields_only'
